from clirainbow.clirainbow import Colorizer
from clirainbow.colors import *


def main():
    c = Colorizer()
    c.print(
        'Welcome to <C><O><L><O><R><I><Z><E><R>',
        BRIGHT_RED,
        BRIGHT_GREEN,
        BRIGHT_BLUE,
        BRIGHT_RED,
        BRIGHT_GREEN,
        BRIGHT_BLUE,
        BRIGHT_RED,
        BRIGHT_GREEN,
        BRIGHT_BLUE
    )


if __name__ == '__main__':
    main()



from colorama import init
# noinspection PyUnresolvedReferences
from clirainbow.formatter import format_color_string, ColorBracketOpenedButNotClosed, MoreBracketsThanColors, MoreColorsThanBrackets


class Colorizer:

    def __init__(self):
        init(autoreset=True)

    def format(self, string, *colors):
        return format_color_string(string, colors)

    def print(self, string, *colors):
        f_string = self.format(string, *colors)
        print(f_string)



from clirainbow.colors import DEFAULT_COLOR

COLOR_START, COLOR_END, CHAR = range(3)

OPENING_BRACKET = '<'
CLOSING_BRACKET = '>'


class ColorBracketOpenedButNotClosed(Exception):

    def __init__(self, position):
        super().__init__(
            f'Color bracket opened at position {position} was never closed')


class MoreBracketsThanColors(Exception):

    def __init__(self, n_brackets, n_colors):
        super().__init__(
            f'You gave me {n_brackets} brackets to fill, but only {n_colors} to fill them with')


class MoreColorsThanBrackets(Exception):

    def __init__(self, n_brackets, n_colors):
        super().__init__(
            f'You gave me {n_colors} colors to use, but only {n_brackets} brackets to fill')


def tokenize(string):
    i = 0
    tokens = []
    in_color = False
    while i < len(string):
        if string[i] == OPENING_BRACKET:
            if string[i:i + 2] == OPENING_BRACKET * 2:
                tokens.append((CHAR, OPENING_BRACKET))
                i += 1
            else:
                tokens.append((COLOR_START, OPENING_BRACKET))
                in_color = True
        elif string[i] == CLOSING_BRACKET:
            if not in_color or string[i:i + 2] == CLOSING_BRACKET * 2:
                tokens.append((CHAR, CLOSING_BRACKET))
                i += 1
            else:
                tokens.append((COLOR_END, CLOSING_BRACKET))
                in_color = False
        else:
            tokens.append((CHAR, string[i]))
        i += 1
    validate(tokens)
    return tokens


def validate(tokens):
    stack = []
    i = 0
    while i < len(tokens):
        if tokens[i][0] == COLOR_START:
            stack.append((COLOR_START, i))
        if tokens[i][0] == COLOR_END:
            stack.pop()
        i += 1
    if stack:
        raise ColorBracketOpenedButNotClosed(stack[-1][1])
    return stack


def format_color_string(string, colors):
    tokens = tokenize(string)
    n_brackets = tokens.count((COLOR_START, OPENING_BRACKET))
    n_colors = len(colors)
    if n_brackets > n_colors:
        raise MoreBracketsThanColors(n_brackets, n_colors)
    if n_colors > n_brackets:
        raise MoreColorsThanBrackets(n_brackets, n_colors)
    i = 0
    color_index = 0
    result = ''
    while i < len(tokens):
        token_type, token_value = tokens[i]
        if token_type == COLOR_START:
            i, parsed = parse_color(i + 1, tokens)
            parsed = colors[color_index] + parsed
            color_index += 1
        else:
            i, parsed = parse_normal(i, tokens)
            parsed = DEFAULT_COLOR + parsed
        result += parsed
    return result


def parse_color(i, tokens):
    parsed = ''
    while i < len(tokens) and tokens[i][0] != COLOR_END:
        parsed += tokens[i][1]
        i += 1
    return i + 1, parsed


def parse_normal(i, tokens):
    parsed = ''
    while i < len(tokens) and tokens[i][0] != COLOR_START:
        parsed += tokens[i][1]
        i += 1
    return i, parsed



from colorama import Fore, Back, Style


def get(key, obj, default):
    if key is None:
        return default
    else:
        return obj.__dict__.get(key.upper(), default)


def _fore(color):
    default = Fore.RESET
    return get(color, Fore, default)


def _back(color):
    default = Back.RESET
    return get(color, Back, default)


def _style(_style):
    default = Style.NORMAL
    return get(_style, Style, default)


def Color(fore=None, back=None, style=None):
    return _fore(fore) + _back(back) + _style(style)


DEFAULT_COLOR = Color()

colors = 'black red green yellow blue magenta cyan white'.split()

brigthnesses = 'bright dim'.split()


def define_color(name, *args, **kwargs):
    globals()[name] = Color(*args, **kwargs)


def color_name(format_string, *args):
    return format_string.format(*map(str.upper, args))


def define_fore_color(color):
    name = color_name('{}', color)
    define_color(name, color)


def define_back_color(color):
    name = color_name('BACK_{}', color)
    define_color(name, back=color)


def define_fore_on_back_color(fore_color, back_color):
    name = color_name('{}_ON_{}', fore_color, back_color)
    define_color(name, fore_color, back_color)


def define_fore_color_with_brightness(fore_color, brightness):
    name = color_name('{}_{}', brightness, fore_color)
    define_color(name, fore_color, style=brightness)


def define_back_color_with_brightness(back_color, brightness):
    name = color_name('{}_BACK_{}', brightness, back_color)
    define_color(name, back=back_color, style=brightness)


def define_fore_color_on_back_color_with_brightness(fore_color, back_color, brightness):
    name = color_name('{}_{}_ON_{}', brightness, fore_color, back_color)
    define_color(name, fore_color, back_color, brightness)


def populate_color_namespace():
    for fore_color in colors:
        define_fore_color(fore_color)
        for back_color in colors:
            define_back_color(back_color)
            define_fore_on_back_color(fore_color, back_color)
            for brightness in brigthnesses:
                define_fore_color_with_brightness(fore_color, brightness)
                define_back_color_with_brightness(back_color, brightness)
                define_fore_color_on_back_color_with_brightness(
                    fore_color, back_color, brightness)


populate_color_namespace()



import os.path
from setuptools import setup

HERE = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(HERE, "README.md")) as readme:
    README = readme.read()

setup(
    name="clirainbow",
    version="1.0.3",
    description="Easily print colored text.",
    long_description=README,
    url="https://github.com/mlevesquedion/clirainbow",
    author="Michaël Lévesque-Dion",
    author_email="mlevesquedion@gmail.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
    ],
    packages=["clirainbow"],
    include_package_data=True,
    install_requires=[
        "colorama"
    ],
)
